/********************************************************************************
 * Project: Statistics for Conference Paper Submissions (CPS)                   *
 * Module : confstat.cpp                                                        *
 * Author : Marcus Hutter (2007) (CopyLeft)                                     *
 * Content: Creates submission/acceptance statistics for conference papers.     *
 * Source : http://www.hutter1.net/ai/confstat.zip                              *
 ********************************************************************************/

#include <stdlib.h>
#include <stdio.h>
#include <string.h>

/********************************************************************************/
/*                        D e s c r i p t i o n                                 */
/********************************************************************************

Quick and dirtly hack for producing submission/acceptance statistics for 
conference papers. The paper data base is read from file paperdb.dat and the 
statistics is written to file confstat.dat. Just compile and call confstat.exe.
No error checking occurs. Increase NBDmax if necessary. See paperdb.dat example
for input format and alt07stat.pdf for the interpretation of the results.
  
Functions:
----------
- InitCPS()            Initialize CPS
- IndexString()        Find index of string or create if necessary:
- ReadStringTable()    Read a table of space-separated strings from file paperdb.dat
- ComputeHistogram()   Histogram of column col of DB[]
- Histo2Freq(void)     Count frequency of values Histo[] to Freq[]
- PerfPerCo()          Compute and print CPS Performance per Column
- PrintCPS()           Compute and print CPS

/********************************************************************************/
/*     S t a t i s t i c s   f o r   C o n f e r e n c e   P a p er s           */
/********************************************************************************/

const int IDBmax=10;           // maximal number of columns
const int NDBmax=3000;         // maximal number of entries (rows) in DB
int   DB[NDBmax][IDBmax];      // relational DataBase
enum COL { paper, score, confidence, author, country, region, field, accepted, pcmember }; // columns
enum COL col;                  // column (used for sorting)
int   IDBread=9;               // number of columns in input file
int   NDBread;                 // read number of entries (rows) in DB
char* NB[IDBmax][NDBmax] = { {"paper"},{"score"},{"confidence"},{"author"},{"country"},{"region"},{"field"},{"accepted"},{"pcmember"} }; // Name Bases
int   nNB[IDBmax];             // number of entries in NB[]
int   SIdx[NDBmax];            // index sorting col of DB
int   Histo[NDBmax+1];         // Histogram
int   Freq[NDBmax+1];          // Frequency count
const int NPAmax=20;           // Maximal number of papers an author can submit
int   PAR[NPAmax+1][NPAmax+1]; // #authors[#accepted][#rejected]
#define max(a,b) ((a)>(b)?(a):(b))

/*------------------------------------------------------------------------------*
  Description:  Initialize CPS
 *-MH---------------------------------------------------------------------------*/
int InitCPS()
/*------------------------------------------------------------------------------*/
{
  int i;
  for(i=0;i<IDBmax;i++) nNB[i]=1; // so that nameindices run from 1 to, 0=header
  NB[accepted][1]=NB[pcmember][1]="No"; 
  NB[accepted][2]=NB[pcmember][2]="Yes"; 
  nNB[accepted]=nNB[pcmember]=3;
  return 1;
} // End of InitCPS()

/*------------------------------------------------------------------------------*
  Description:  Find index of string or create if necessary:
                Append name to nb[] if not alread in nb[] and return index of name 
 *-MH---------------------------------------------------------------------------*/
int IndexString(char* nb[], int &nnb, char* name)
/*------------------------------------------------------------------------------*/
{
  int i;
  for(i=0;i<nnb;i++) if (!strcmp(nb[i],name)) break;
  if (i==nnb) { nb[i]=(char*)malloc(strlen(name)+1); strcpy(nb[i],name); nnb++; }
  return i;
} // End of IndexString()

/*------------------------------------------------------------------------------*
  Description:  Read a table of space-separated strings from file paperdb.dat
                into data based DB[][] and create name base NB[]
 *-MH---------------------------------------------------------------------------*/
void ReadStringTable(void)
/*------------------------------------------------------------------------------*/
{
  int i,k,flag=1;
  char s[999];
  FILE *devr=fopen("paperdb.dat","rt");

  for(k=0;flag>0;k++)
  { for(i=0;i<IDBread;i++)
    { flag=fscanf(devr,"%s",s);
      if (flag<=0) break;
      if (s[0]=='%'||s[0]=='/') { fgets(s,999,devr); k--; break; }
      DB[k][i]=IndexString(NB[i],nNB[i],s);
    } 
  }
  NDBread=k-1; 
  fclose(devr);
} // End of ReadStringTable()

/*------------------------------------------------------------------------------*
  Description:  Histogram of column col of DB[]
  Return Value: Histo[i] <- #{DB[][col]==i}
 *-MH---------------------------------------------------------------------------*/
void ComputeHistogram(void)
/*------------------------------------------------------------------------------*/
{
  for(int i=0;i<nNB[col];i++) Histo[i]=0;
  for(int k=0;k<NDBread;k++) Histo[DB[k][col]]++;
} // End of ComputeHistogram()

/*------------------------------------------------------------------------------*
  Description:  Count frequency of values Histo[0..nNB[col]-1] to Freq[0..NDBread]
 *-MH---------------------------------------------------------------------------*/
void Histo2Freq(void)
/*------------------------------------------------------------------------------*/
{
  for(int k=0;k<=NDBread;k++) Freq[k]=0;
  for(int i=0;i<nNB[col];i++) Freq[Histo[i]]++;
} // End of Histo2Freq()

/*------------------------------------------------------------------------------*
  Description:  Compute and print CPS Performance per (col=Country or Region) Column
 *-MH---------------------------------------------------------------------------*/
void PerfPerCo(FILE *devw)
/*------------------------------------------------------------------------------*/
{ int i,k;
  int NPCM[NDBmax+1]; // number of PCmembers
  fprintf(devw,"\n%20s: Subm(A*P) Acc(A*P,Rate) PCmembers\n",NB[col][0]);
  for(i=0;i<nNB[col];i++) { Histo[i]=0; Freq[i]=0; NPCM[i]=0; }
  for(k=0;k<NDBread;k++) 
  { Freq[DB[k][col]]++;
    if (DB[k][accepted]==2) Histo[DB[k][col]]++;
  }
  int TT[NDBmax+1]; // temp-array
  for(k=0;k<NDBread;k++) TT[DB[k][author]]=k; // DB[k][pcmember]
  for(i=1;i<nNB[author];i++) if (DB[TT[i]][pcmember]==2) NPCM[DB[TT[i]][col]]++; 
  for(i=1;i<nNB[col];i++) fprintf(devw,"%20s: %4d %7d %7d%% %4d\n",NB[col][i],Freq[i],Histo[i],(100*Histo[i])/Freq[i],NPCM[i]);
} // End of PerfPerCo()

/*------------------------------------------------------------------------------*
  Description:  Compute and print CPS
 *-MH---------------------------------------------------------------------------*/
void PrintCPS(void)
/*------------------------------------------------------------------------------*/
{
  int i,k;
  FILE *devw=fopen("confstat.dat","wt");
  fprintf(devw,"///////////////////////////////////////////////////////////////\n");
  fprintf(devw,"//    Conference Paper Submission Statistics                 //\n");
  fprintf(devw,"//    created from paperdb.dat by                            //\n");
  fprintf(devw,"//    http://www.hutter1.net/ai/confstat.zip                 //\n");
  fprintf(devw,"///////////////////////////////////////////////////////////////\n");

  // accepted/rejected papers per author
  fprintf(devw,"\nAuthors submitting papers\n");
  int nap=0,nrp=0; // maximal number of accepted/rejected papers
  col=author;
  for(i=0;i<nNB[col];i++) { Histo[i]=0; Freq[i]=0; }
  for(k=0;k<NDBread;k++) 
  { if (DB[k][accepted]==2) { Freq[DB[k][col]]++;  nap=max(nap,Freq[DB[k][col]]);  }
    else                    { Histo[DB[k][col]]++; nrp=max(nrp,Histo[DB[k][col]]); }
  }
  for(k=0;k<=NPAmax;k++) for(i=0;i<=NPAmax;i++) PAR[k][i]=0;
  for(i=1;i<nNB[col];i++) PAR[Freq[i]][Histo[i]]++;

  fprintf(devw,"re\\ac"); for(k=0;k<=nap;k++) fprintf(devw,"%5d",k);
  fprintf(devw,"\n");
  for(i=0;i<=nrp;i++)
  { fprintf(devw,"%5d",i);
    for(k=0;k<=nap;k++) fprintf(devw,"%5d",PAR[k][i]);
    fprintf(devw,"\n");
  }

  // authors per paper
  col=author;
  ComputeHistogram();
  Histo2Freq();
  fprintf(devw,"\n#Authors per Paper Statistic:\n");
  fprintf(devw,"#Authors    1   2   3   4   5   6   7   8   9\n");
  fprintf(devw,"#Papers   %3d %3d %3d %3d %3d %3d %3d %3d %3d\n",Freq[1],Freq[2],Freq[3],Freq[4],Freq[5],Freq[6],Freq[7],Freq[8],Freq[9]);

  // accepance rates
  fprintf(devw,"\nAcceptance Rates:\n");
  fprintf(devw,"Category    Submitted Accepted Rejected Rate\n");

  for(k=0;k<NDBread;k++) Histo[DB[k][author]]=DB[k][accepted];
  col=author; Histo2Freq();
  if (Freq[1]+Freq[2]!=nNB[author]-1) printf("#accepted+#rejected <> total#papers\n"); 
  fprintf(devw,"Authors     %7d %7d %7d %7d%%\n",nNB[author]-1,Freq[2],Freq[1],(100*Freq[2])/(nNB[author]-1));

  for(k=0;k<NDBread;k++) Histo[DB[k][paper]]=DB[k][accepted];
  col=paper; Histo2Freq();
  if (Freq[1]+Freq[2]!=nNB[paper]-1) printf("#accepted+#rejected <> total#papers\n"); 
  fprintf(devw,"Papers      %7d %7d %7d %7d%%\n",nNB[paper]-1,Freq[2],Freq[1],(100*Freq[2])/(nNB[paper]-1));

  col=accepted;
  ComputeHistogram();
  if (Histo[1]+Histo[2]!=NDBread) printf("#accepted+#rejected <> total#papers\n"); 
  fprintf(devw,"Authors*Papers %4d %7d %7d %7d%%\n",NDBread,Histo[2],Histo[1],(100*Histo[2])/NDBread);

  fprintf(devw,"Authors/Paper%7.2lf %7.2lf %7.2lf\n",(double)NDBread/(nNB[paper]-1),(double)Histo[2]/Freq[2],(double)Histo[1]/Freq[2]);

  // Papers by fields/areas (todo: also list out of how many rejected papers)
  fprintf(devw,"\nAccepted papers only. #papers and A*P\n");
  col=field;
  for(i=0;i<nNB[col];i++) { Histo[i]=0; Freq[i]=0; }
  for(k=0;k<NDBread;k++) if (DB[k][accepted]==2) 
  { Histo[DB[k][col]]++;
    if (k==0 || DB[k][paper]!=DB[k-1][paper]) Freq[DB[k][col]]++; // assumes same paper is consecutive
  }
  for(i=1;i<nNB[col];i++) fprintf(devw,"%20s: %4d %7d\n",NB[col][i],Freq[i],Histo[i]);

  // Country Performance
  col=country; 
  PerfPerCo(devw);

  // Performance per Region
  col=region;
  PerfPerCo(devw);

  // print paperdb.dat
  fprintf(devw,"\n// Control Copy of Conference Paper Submission Data Base paperdb.dat\n");
  for(i=0;i<IDBread;i++) fprintf(devw,"%-12s ",NB[i][0]);
  fprintf(devw,"\n");

  for(k=0;k<NDBread;k++)
  { for(i=0;i<IDBread;i++) fprintf(devw,"%-12s ",NB[i][DB[k][i]]);
    fprintf(devw,"\n");
  } 

  fclose(devw);
} // End of PrintCPS()


/*------------------------------------------------------------------------------*
  Description:  main() 
 *-MH---------------------------------------------------------------------------*/
int main()
/*------------------------------------------------------------------------------*/
{
  printf("Reading paperdb.dat and writing statistic to confstat.dat ... ");
  InitCPS();
  ReadStringTable();
  PrintCPS();
  printf("done. Press any key to exit.\n");
  getchar();
  return 0;
} // End of main()

/*---------------------------End-of-confstat.c----------------------------------*/
